import logging
import boto3

from spaceone.core.error import *
from spaceone.file_manager.connector.file_base_connector import FileBaseConnector

__all__ = ['AWSS3Connector']
_LOGGER = logging.getLogger(__name__)


class AWSS3Connector(FileBaseConnector):

    def __init__(self, transaction, config):
        super().__init__(transaction, config)

        self.client = None
        self.bucket_name = None
        self._create_session()
        self._set_bucket()

    def _create_session(self):
        aws_access_key_id = self.config.get('aws_access_key_id')
        aws_secret_access_key = self.config.get('aws_secret_access_key')
        region_name = self.config.get('region_name')

        if region_name is None:
            raise ERROR_CONNECTOR_CONFIGURATION(backend='AWSS3Connector')

        if aws_access_key_id and aws_secret_access_key:
            self.client = boto3.client('s3', aws_access_key_id=aws_access_key_id,
                                       aws_secret_access_key=aws_secret_access_key, region_name=region_name)
        else:
            self.client = boto3.client('s3', region_name=region_name)

    def _set_bucket(self):
        bucket_name = self.config.get('bucket_name')

        if bucket_name is None:
            raise ERROR_CONNECTOR_CONFIGURATION(backend='AWSS3Connector')

        self.bucket_name = bucket_name

    def get_upload_url(self, file_id, file_name):
        object_name = self._generate_object_name(file_id, file_name)
        response = self.client.generate_presigned_post(self.bucket_name, object_name)
        return response['url'], response['fields']

    def get_download_url(self, file_id, file_name):
        object_name = self._generate_object_name(file_id, file_name)
        response = self.client.generate_presigned_url('get_object',
                                                      Params={'Bucket': self.bucket_name, 'Key': object_name})

        return response

    def check_file(self, file_id, file_name):
        object_name = self._generate_object_name(file_id, file_name)
        try:
            self.client.get_object(Bucket=self.bucket_name, Key=object_name)
            return True
        except Exception as e:
            _LOGGER.debug(f'[check_file] get_object error: {e}')
            return False

    def delete_file(self, file_id, file_name):
        object_name = self._generate_object_name(file_id, file_name)
        self.client.delete_object(Bucket=self.bucket_name, Key=object_name)

    @staticmethod
    def _generate_object_name(file_id, file_name):
        return f'{file_id}/{file_name}'



import logging
import functools
from spaceone.api.file_manager.v1 import file_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.file_manager.model.file_model import File, FileReference


__all__ = ['FileInfo', 'FilesInfo']
_LOGGER = logging.getLogger(__name__)


def FileReferenceInfo(reference_vo: FileReference):
    if reference_vo:
        info = {
            'resource_type': reference_vo.resource_type,
            'resource_id': reference_vo.resource_id
        }

        return file_pb2.FileReference(**info)
    else:
        return None


def FileInfo(file_vo: File, minimal=False, upload_url=None, upload_options=None, download_url=None):
    info = {
        'file_id': file_vo.file_id,
        'name': file_vo.name,
        'state': file_vo.state,
        'reference': file_vo.reference,
        'scope': file_vo.scope,
        'domain_id': file_vo.domain_id
    }

    if minimal is False:
        info.update({
            'file_type': file_vo.file_type,
            'tags': change_struct_type(file_vo.tags),
            'reference': FileReferenceInfo(file_vo.reference),
            'user_id': file_vo.user_id,
            'user_domain_id': file_vo.user_domain_id,
            'created_at': utils.datetime_to_iso8601(file_vo.created_at)
        })

        if upload_url:
            info['upload_url'] = upload_url
            info['upload_options'] = change_struct_type(upload_options) if upload_options else None

        if download_url:
            info['download_url'] = download_url

    return file_pb2.FileInfo(**info)


def FilesInfo(file_vos, total_count, **kwargs):
    results = list(map(functools.partial(FileInfo, **kwargs), file_vos))

    return file_pb2.FilesInfo(results=results, total_count=total_count)



import logging
from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel

_LOGGER = logging.getLogger(__name__)


class FileReference(EmbeddedDocument):
    resource_type = StringField(required=True)
    resource_id = StringField(required=True)

    def to_dict(self):
        return dict(self.to_mongo())


class File(MongoModel):
    file_id = StringField(max_length=40, generate_id='file', unique=True)
    name = StringField(max_length=255, required=True)
    state = StringField(max_length=40, choices=('PENDING', 'DONE'), default='PENDING')
    scope = StringField(max_length=40, choices=('PUBLIC', 'DOMAIN'), required=True)
    file_type = StringField(max_length=255, null=True, default=None)
    tags = DictField()
    reference = EmbeddedDocumentField(FileReference, null=True, default=None)
    domain_id = StringField(max_length=40, null=True, default=None)
    user_id = StringField(max_length=40)
    user_domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'state',
            'tags',
            'reference'
        ],
        'minimal_fields': [
            'file_id',
            'name',
            'state',
            'reference'
        ],
        'change_query_keys': {
            'resource_type': 'reference.resource_type',
            'resource_id': 'reference.resource_id',
            'user_domains': 'domain_id'
        },
        'ordering': [
            'name'
        ],
        'indexes': [
            'state',
            'scope',
            'file_type',
            'reference.resource_type',
            'reference.resource_id',
            'domain_id',
            'user_id',
            'user_domain_id'
        ]
    }



import logging

from spaceone.core.service import *
from spaceone.file_manager.error import *
from spaceone.file_manager.model.file_model import File
from spaceone.file_manager.manager.file_manager import FileManager
from spaceone.file_manager.manager.file_connector_manager import FileConnectorManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class FileService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.file_mgr: FileManager = self.locator.get_manager('FileManager')

    @transaction(append_meta={
        'authorization.scope': 'PUBLIC_OR_DOMAIN',
        'authorization.require_domain_id': True
    })
    @check_required(['name'])
    def add(self, params):
        """ Add file

        Args:
            params (dict): {
                'name': 'str',
                'tags': 'dict',
                'reference': 'dict',
                'project_id': 'str',
                'domain_id': 'str,
                'user_id': 'str', // from meta
                'user_domain_id': 'str' // from meta
            }

        Returns:
            file_vo
        """

        domain_id = params.get('domain_id')
        params['user_id'] = self.transaction.get_meta('user_id')
        params['user_domain_id'] = self.transaction.get_meta('domain_id')

        if domain_id:
            params['scope'] = 'DOMAIN'
        else:
            params['scope'] = 'PUBLIC'

        params['file_type'] = self._get_file_type(params['name'])

        file_vo: File = self.file_mgr.create_file(params)

        file_conn_mgr: FileConnectorManager = self.locator.get_manager('FileConnectorManager')
        upload_url, upload_options = file_conn_mgr.get_upload_url(file_vo.file_id, file_vo.name)

        return file_vo, upload_url, upload_options

    @transaction(append_meta={
        'authorization.scope': 'PUBLIC_OR_DOMAIN'
    })
    @check_required(['file_id'])
    def update(self, params):
        """ Update file

        Args:
            params (dict): {
                'file_id': 'str',
                'tags': 'dict',
                'reference': 'dict',
                'domain_id': 'str',
                'user_domains': 'list' // from meta
            }

        Returns:
            file_vo
        """

        file_id = params['file_id']
        user_domains = self.transaction.get_meta('user_domains')

        file_vo: File = self.file_mgr.get_file(file_id, user_domains)
        file_vo = self.file_mgr.update_file_by_vo(params, file_vo)

        return file_vo

    @transaction(append_meta={
        'authorization.scope': 'PUBLIC_OR_DOMAIN'
    })
    @check_required(['file_id'])
    def delete(self, params):
        """ Delete file

        Args:
            params (dict): {
                'file_id': 'str',
                'domain_id': 'str',
                'user_domains': 'list' // from meta
            }

        Returns:
            None
        """

        file_id = params['file_id']
        user_domains = self.transaction.get_meta('user_domains')

        file_vo: File = self.file_mgr.get_file(file_id, user_domains)

        file_conn_mgr: FileConnectorManager = self.locator.get_manager('FileConnectorManager')
        file_conn_mgr.delete_file(file_id, file_vo.name)

        self.file_mgr.delete_file_by_vo(file_vo)

    @transaction(append_meta={
        'authorization.scope': 'PUBLIC_OR_DOMAIN'
    })
    @check_required(['file_id'])
    def get_download_url(self, params):
        """ Get download url of file

        Args:
            params (dict): {
                'file_id': 'str',
                'domain_id': 'str',
                'user_domains': 'list' // from meta
            }

        Returns:
            file_data (dict)
        """

        file_id = params['file_id']
        user_domains = self.transaction.get_meta('user_domains')

        file_vo: File = self.file_mgr.get_file(file_id, user_domains)

        file_conn_mgr: FileConnectorManager = self.locator.get_manager('FileConnectorManager')

        if file_vo.state == 'PENDING':
            if not file_conn_mgr.check_file(file_id, file_vo.name):
                raise ERROR_FILE_UPLOAD_STATE()

            file_vo = self.file_mgr.update_file_by_vo({'state': 'DONE'}, file_vo)

        download_url = file_conn_mgr.get_download_url(file_id, file_vo.name, file_vo.domain_id)

        return file_vo, download_url

    @transaction(append_meta={
        'authorization.scope': 'PUBLIC_OR_DOMAIN'
    })
    @check_required(['file_id'])
    def get(self, params):
        """ Get file

        Args:
            params (dict): {
                'file_id': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            file_vo
        """

        file_id = params['file_id']
        user_domains = self.transaction.get_meta('user_domains')

        return self.file_mgr.get_file(file_id, user_domains, params.get('only'))

    @transaction(append_meta={
        'authorization.scope': 'PUBLIC_OR_DOMAIN'
    })
    @append_query_filter(['file_id', 'name', 'state', 'scope', 'file_type', 'resource_type', 'resource_id',
                          'user_domain_id', 'domain_id', 'user_domains'])
    @append_keyword_filter(['file_id', 'name'])
    def list(self, params):
        """ List files

        Args:
            params (dict): {
                'file_id': 'str',
                'name': 'str',
                'state': 'str',
                'scope': 'str',
                'file_type': 'str',
                'resource_type': 'str',
                'resource_id': 'str',
                'user_domain_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)',
                'user_domains': 'list', // from meta
            }

        Returns:
            results (list)
            total_count (int)
        """

        query = params.get('query', {})
        return self.file_mgr.list_files(query)

    @transaction(append_meta={
        'authorization.scope': 'PUBLIC_OR_DOMAIN'
    })
    @check_required(['query'])
    @append_query_filter(['domain_id', 'user_domains'])
    @append_keyword_filter(['file_id', 'name'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_domains': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.file_mgr.stat_files(query)

    @staticmethod
    def _get_file_type(file_name):
        file_name_split = file_name.split('.')
        if len(file_name_split) == 1:
            return None
        else:
            return file_name_split[-1]



from spaceone.core.error import *


class ERROR_NOT_DEFINED_FILE_BACKEND(ERROR_BASE):
    _message = 'File backend not defined. (backend = {backend})'


class ERROR_FILE_UPLOAD_STATE(ERROR_INVALID_ARGUMENT):
    _message = 'The file has not been uploaded yet. (state = PENDING)'



from spaceone.api.file_manager.v1 import file_pb2, file_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class File(BaseAPI, file_pb2_grpc.FileServicer):

    pb2 = file_pb2
    pb2_grpc = file_pb2_grpc

    def add(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('FileService', metadata) as file_service:
            file_vo, upload_url, upload_options = file_service.add(params)
            return self.locator.get_info('FileInfo', file_vo, upload_url=upload_url, upload_options=upload_options)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('FileService', metadata) as file_service:
            return self.locator.get_info('FileInfo', file_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('FileService', metadata) as file_service:
            file_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def get_download_url(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('FileService', metadata) as file_service:
            file_vo, download_url = file_service.get_download_url(params)
            return self.locator.get_info('FileInfo', file_vo, download_url=download_url)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('FileService', metadata) as file_service:
            return self.locator.get_info('FileInfo', file_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('FileService', metadata) as file_service:
            file_vos, total_count = file_service.list(params)
            return self.locator.get_info('FilesInfo', file_vos, total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('FileService', metadata) as file_service:
            return self.locator.get_info('StatisticsInfo', file_service.stat(params))



DATABASE_AUTO_CREATE_INDEX = True
DATABASES = {
    'default': {
        'db': 'file-manager',
        'host': 'localhost',
        'port': 27017,
        'username': '',
        'password': ''
    }
}

CACHES = {
    'default': {},
    'local': {
        'backend': 'spaceone.core.cache.local_cache.LocalCache',
        'max_size': 128,
        'ttl': 300
    }
}

HANDLERS = {}

BACKEND = 'AWSS3Connector'
CONNECTORS = {
    'AWSS3Connector': {
        'backend': 'spaceone.file_manager.connector.aws_s3_connector.AWSS3Connector',
        'aws_access_key_id': '<optional>',
        'aws_secret_access_key': '<optional>',
        'region_name': '<required>',
        'bucket_name': '<required>',
    },
    'SpaceConnector': {
        'backend': 'spaceone.core.connector.space_connector.SpaceConnector',
        'endpoints': {
            'identity': 'grpc://identity:50051',
        }
    },
}



from spaceone.file_manager.manager.file_manager import FileManager
from spaceone.file_manager.manager.file_connector_manager import FileConnectorManager
from spaceone.file_manager.manager.identity_manager import IdentityManager
